package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import javafx.geometry.Pos;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.shape.Line;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FactoryDecorator implements IsObservable{
    private Factory factory;
    private FileIO fileIO;
    private Pane pane;
    private List<IsObserver> listener = new ArrayList<>();

    public FactoryDecorator(FileIO fileIO, Factory factory, Pane pane){
        setFactory(factory);
        setFileIO(fileIO);
        setPane(pane);
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setFactory(Factory factory) {
        this.factory = factory;
    }

    public void setPane(Pane pane) {
        this.pane = pane;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }

    public void openTournament(FileIO.TournamentFile tournamentFile){
        try {
            factory.setTournament(fileIO.loadTournament(tournamentFile));
            openScheduleView();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } //TODO handle and logging
    }

    public void openTournamentList() {
        factory.loadTournamentList((BorderPane) pane, this);
        informListener();
    }

    public void openParticipantFormular() {
        factory.loadParticipantFormular((BorderPane) pane, this);
        informListener();
    }

    public void openPlacesFormular() {
        factory.loadPlacesFormular((BorderPane) pane, this);
        informListener();
    }

    public void openScheduleView() {
        factory.loadGameScheduler((BorderPane) pane, this);
        informListener();
    }

    public void loadGameList(HBox hBoxCenter, TournamentDecorator tournamentDecorator, boolean treeView) {
        hBoxCenter.getChildren().clear();

        List<List<Game>> gameList = tournamentDecorator.getTournament().getGameList();

        List<GameDecorator> gameDecoratorsList = new ArrayList<>();
        double gameBoxHeight = 0;
        double spacingFactor = 0;
        for (int i = 0; i < gameList.size(); i++) {
            List<GameDecorator> newGameDecoratorsList = new ArrayList<>();
            VBox vBox = new VBox();
            if(treeView){
                vBox.setAlignment(Pos.CENTER);
                vBox.setSpacing(gameBoxHeight * spacingFactor);
            } else {
                vBox.setAlignment(Pos.TOP_CENTER);
            }
            spacingFactor = spacingFactor * 2 + 1;
            for (int j = 0; j < gameList.get(i).size(); j++) {
                GameDecorator gameDecorator = new GameDecorator(gameList.get(i).get(j));
                newGameDecoratorsList.add(gameDecorator);
                GameController gameController = openGameView(vBox,gameDecorator);
                if(i>0){
                    gameController.addListener(gameDecoratorsList.get(j*2));
                    gameController.addListener(gameDecoratorsList.get(j*2+1));
                } else if(gameBoxHeight == 0) {
                    gameBoxHeight = gameController.getGameBoxHeigth();
                }
                gameDecorator.addListener(new IsObserver() {
                    @Override
                    public void update() {
                        tournamentDecorator.saveTournament();
                    }
                });
                gameController.loadContent();
            }
            hBoxCenter.getChildren().add(vBox);
            if(treeView){
                if(i+1 < gameList.size())
                hBoxCenter.getChildren().add(drawLines(vBox, gameBoxHeight, 30));
            }
            gameDecoratorsList = newGameDecoratorsList;
        }
    }

    public VBox drawLines(VBox gameVBox, double gameBoxHeight, double lineLength){
        VBox completeLineVBox = new VBox();
        completeLineVBox.setAlignment(Pos.CENTER_LEFT);
        double lineSpacing = gameVBox.getSpacing() + gameBoxHeight - 1;
        completeLineVBox.setSpacing(lineSpacing);
        for(int i = 0; i < gameVBox.getChildren().size(); i+=2){
            HBox lineBox = new HBox();
            lineBox.setAlignment(Pos.CENTER);

            //add Lines from left Game to center
            VBox vBox = new VBox();
            vBox.setSpacing(lineSpacing);
            vBox.getChildren().add(new Line(0,0,lineLength,0));
            vBox.getChildren().add(new Line(0,0,lineLength,0));
            lineBox.getChildren().add(vBox);


            //add vertical line
            lineBox.getChildren().add(new Line(0, 0, 0, lineSpacing));

            //add line to right game
            lineBox.getChildren().add(new Line(0, 0, lineLength, 0));

            completeLineVBox.getChildren().add(lineBox);


        }
        return completeLineVBox;
    }

    public GameController openGameView(VBox vBox, GameDecorator gameDecorator) {
        return factory.loadGameView(vBox ,gameDecorator, this);
    }

    public void printMessageToFooter(String msg, boolean error) {
        factory.printMessageToFooter((BorderPane) pane,msg,error);
    }

    public void clearMessage(boolean error){
        factory.resetFooter((BorderPane) pane, error);
    }

    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }
}
